package aisl.model;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.TreeMap;

/**
 * Access facade to the Forum.
 *
 * @author rosmord
 */
public class ForumFacade implements Serializable {

    /**
     * Change if needed.
     */
    private static final String FORUM_PATH = "forumBis.dat";
    /**
     * The file where the forum is saved.
     */
    private File file;
    private TreeMap<Integer, Message> messages = new TreeMap<Integer, Message>();
    private HashMap<String, User> users = new HashMap<String, User>();
    private int maxMessageNumber = 0;
    /**
     * Create the only instance for the forum. (not final, because of
     * administrative methods clear and initDb)
     */
    private static ForumFacade instance = read(FORUM_PATH);

    public static ForumFacade getInstance() {
        return instance;
    }

    /**
     * Create a forumFacade, reading it from a file (or from scratch if there is
     * no file).
     *
     * @param path
     * @return
     */
    private static ForumFacade read(String path) {
        File file = new File(path);
        ForumFacade f = new ForumFacade(file);
        try {
            if (file.exists()) {
                ObjectInputStream o = new ObjectInputStream(new FileInputStream(file));
                try {
                    f = (ForumFacade) o.readObject();
                    // update the file field (even if it has moved).
                    f.file = file;
                } finally {
                    o.close();
                }
            } else {
                // Pour les besoins de l'exercice, on ajoute quelques messages
                User admin = f.createUser("admin", "admin", Group.ADMIN);
                User toto = f.createUser("toto", "toto", Group.USER);
                f.createMessage("premier message", "ceci est le texte du premier message", "admin");
                f.createMessage("second message", "ceci est le texte du second message", "toto");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return f;
    }

    private ForumFacade(File fichier) {
        this.file = fichier;
    }

    /**
     * Method for creating a user.
     * <p> Only legal way to create the user object.
     *
     * @param login
     * @param password
     * @param group
     * @return
     * @throws ForumException
     */
    public User createUser(String login, String password, Group group) throws ForumException {
        if (users.get(login) != null) {
            throw new ForumException("User already exists " + login);
        }
        User utilisateur = new User(login, password, group);
        users.put(login, utilisateur);
        save();
        return utilisateur;
    }

    /**
     * Update the user object in the forum.
     *
     * @param login
     * @param password
     * @param groupe
     * @throws ForumException
     */
    public void updateUser(String login, String password, Group groupe) throws ForumException {
        User user = users.get(login);
        if (user == null) {
            throw new ForumException("Unknown user " + login);
        } else {
            user.setPassword(password);
            user.setGroup(groupe);
            save();
        }
    }

    /**
     * Gets all users.
     *
     * @return
     */
    public Collection<User> getUsers() {
        return new ArrayList<User>(users.values());
    }

    /**
     * Creates a new Message.
     *
     * @param title
     * @param text
     * @param authorLogin
     * @return
     */
    public synchronized int createMessage(String title, String text, String authorLogin) {
        maxMessageNumber++;
        int pos = maxMessageNumber;
        Message message = new Message(pos, title, text, getUser(authorLogin));
        messages.put(maxMessageNumber, message);
        save();
        return pos;
    }

    /**
     * Updates the characteristics of an existing message.
     *
     * @param id
     * @param title
     * @param text
     *
     */
    public synchronized void updateMessage(int id, String title, String text) throws ForumException {
        Message msg= messages.get(id);
        if (msg != null) {
            msg.setTitle(title);
            msg.setText(text);
            save();
        } else {
            throw new ForumException(("Message " + id + " doesn't exist"));
        }
    }

    /**
     * Saves the forum data to file (call after any change in the forum)
     */
    private synchronized void save() {
        try {
            ObjectOutputStream o = new ObjectOutputStream(new FileOutputStream(FORUM_PATH));
            try {
                o.writeObject(this);
            } finally {
                o.close();
            }
        } catch (IOException iOException) {
            throw new RuntimeException(iOException);
        }
    }

    /**
     * Returns a message, knowing its id.
     *
     * @param id
     * @return
     */
    public synchronized Message getMessage(int id) {
        return messages.get(id);
    }

    public synchronized Collection<Message> getAllMessages() {
        return new ArrayList<Message>(messages.values());
    }

    /**
     * Deletes the whole database.
     *
     * <p> Use only for administration, when classes have changed and the old
     * file content is incorrect.
     */
    public static synchronized void clear() {
        new File(FORUM_PATH).delete();
    }

    /**
     * Creates a new database after a clear. (normally, only use for setting up
     * the tests)
     */
    public static synchronized void clearAndReinit() {
        clear();
        instance = read(FORUM_PATH);
    }

    /**
     * Returns a given user.
     *
     * @param login
     * @return
     */
    public synchronized User getUser(String login) {
        return users.get(login);
    }
}
